@@ -2,18 +2,27 @@
 from django.contrib.auth import get_user
 from django.contrib.auth.models import User
 from django.core.urlresolvers import reverse
-from django.http import HttpResponseRedirect, HttpResponse
-from django.test.client import Client as TestClient
+from django.http import (HttpResponseRedirect, HttpResponse, 
+    HttpResponseForbidden, HttpResponseBadRequest)
+from django.test.client import Client as TestClient, RequestFactory
 from django.test.testcases import TestCase
 from django.utils import simplejson
+from simple_sso.server import AuthorizeView
+from simple_sso.signatures import build_signature
 from simple_sso.sso_client.utils import load_json_user
 from simple_sso.sso_server.models import Client, Token
+from simple_sso.test_urls import test_server
 from simple_sso.test_utils.context_managers import (SettingsOverride, 
     UserLoginContext)
 from simple_sso.utils import SIMPLE_KEYS, gen_secret_key
+import datetime
 import urlparse
 
+
+
 class SimpleSSOTests(TestCase):
+    urls = 'simple_sso.test_urls'
+
     def setUp(self):
         import requests
         def get(url, params={}, headers={}, cookies=None, auth=None, **kwargs):
@@ -121,3 +130,71 @@ class SimpleSSOTests(TestCase):
         for key in SIMPLE_KEYS:
             self.assertEqual(getattr(user, key), userdata[key])
         self.assertFalse(user.check_password('testpassword'))
+    
+    def test_request_token_view_invalid_signature(self):
+        client = Client.objects.create(root_url='/client/')
+        data = {'key': client.key, 'signature': 'x' * 64}
+        response = self.client.get(reverse('simple-sso-request-token'), data)
+        self.assertEqual(response.status_code, HttpResponseForbidden.status_code)
+    
+    def test_request_token_view_bad_request(self):
+        data = {'key': 'y' * 64, 'signature': 'x' * 64}
+        response = self.client.get(reverse('simple-sso-request-token'), data)
+        self.assertEqual(response.status_code, HttpResponseBadRequest.status_code)
+    
+    def test_authorize_view_token_timeout(self):
+        client = Client.objects.create(root_url='/client/')
+        token = Token.objects.create_for_client(client)
+        token.created = datetime.datetime.now() - test_server.token_timeout - datetime.timedelta(hours=1) 
+        token.save()
+        data = {
+            'key': client.key,
+            'request_token': token.request_token,
+        }
+        data['signature'] = build_signature(data.items(), client.secret)
+        response = self.client.get(reverse('simple-sso-authorize'), data)
+        self.assertEqual(response.status_code, HttpResponseForbidden.status_code)
+    
+    def test_authorize_view_invalid_request_token(self):
+        client = Client.objects.create(root_url='/client/')
+        data = {
+            'key': client.key,
+            'request_token': 'x' * 64,
+        }
+        data['signature'] = build_signature(data.items(), client.secret)
+        response = self.client.get(reverse('simple-sso-authorize'), data)
+        self.assertEqual(response.status_code, HttpResponseBadRequest.status_code)
+    
+    def test_authorize_view_invalid_signature(self):
+        client = Client.objects.create(root_url='/client/')
+        token = Token.objects.create_for_client(client)
+        token.created = datetime.datetime.now() - test_server.token_timeout - datetime.timedelta(hours=1) 
+        token.save()
+        data = {
+            'key': client.key,
+            'request_token': token.request_token,
+            'signature': 'x' * 64
+        }
+        response = self.client.get(reverse('simple-sso-authorize'), data)
+        self.assertEqual(response.status_code, HttpResponseForbidden.status_code)
+    
+    def test_authorize_view_no_access(self):
+        class NoAccessAuthorizeView(AuthorizeView):
+            server = test_server
+            
+            def has_access(self):
+                return False
+        client = Client.objects.create(root_url='/client/')
+        token = Token.objects.create_for_client(client)
+        data = {
+            'key': client.key,
+            'request_token': token.request_token,
+        }
+        data['signature'] = build_signature(data.items(), client.secret)
+        request = RequestFactory().get(reverse('simple-sso-authorize'), data)
+        USERNAME = PASSWORD = 'myuser'
+        server_user = User.objects.create_user(USERNAME, 'my@user.com', PASSWORD)
+        request.user = server_user
+        view = NoAccessAuthorizeView.as_view()
+        response = view(request)
+        self.assertEqual(response.status_code, HttpResponseForbidden.status_code)
